/*

 jobpipe_c

 Routine to connect two	(not yet activated) jobs via a pipe.
 Each end of the pipe is given to its respective job.
 Returns -1 for	error, else returns zero, sets errno and _oserr.

 Date		Author		Comments
 ----		------		--------
 02/09/1991	P.J. Taylor	Original.
 08/09/1992	P.J. Taylor	Amended.

 1.04
 ----
 07/03/1998	A.D.Ives	Source code reformatted.

*/

#include <sys/types.h>
#include <errno.h>
#include <qdos.h>
#include <debug.h>
#include <string.h>
#include "libpjt.h"

int jobpipe(jobid_t frjob, jobid_t tojob) {

  /* frjob	Job to write to	the pipe	*/
  /* tojob	Job to read from the pipe	*/

  static char pn[]= "PIPE_" ;
  char fn[11] ,			/* Filename for	pipe open		*/
       psz[6] ,			/* Temporary to	hold pipe length	*/
       *p ;
  chanid_t frchan ,		/* Channel for frjob			*/
	   tochan ;		/* Channel for tojob			*/
  int x	;
  long o_oserr ;		/* Saved error				*/
  int o_errno ;			/* Saved error				*/

  p= &psz[5] ;
  *p= 0	;
  for (	x= _pipesize ; p > psz && x > 0	; x/= 10 )
    *--p= (char) ((int)	'0' + x	% 10) ;
  strcpy(fn, pn) ;
  strcat(fn, p)	;		/* fn =	"PIPE_nnnn" */
  
  if ( !chanopn(&frchan, fn, frjob, 0) && chanopn(&tochan, pn, tojob, frchan) )	{
    o_errno= errno ;
    o_oserr= _oserr ;
    io_close(frchan) ;
    errno= o_errno ;
    _oserr= o_oserr ;
  }

  if ( !errno && ( jobchan(frjob, frchan) || jobchan(tojob, tochan) ) )	{
    o_errno= errno ;
    o_oserr= _oserr ;
    io_close(tochan) ;
    io_close(frchan) ;
    errno= o_errno ;
    _oserr= o_oserr ;
  }

  if ( errno )
    return -1 ;
  else
    return 0 ;
}
