/*

 jobstrt_c

 Routine to activate a (not yet	activated) job previously loaded by
 loadjob. If the job is	already	started	an error is returned.
 Returns -1 for	error, else zero .
 Sets errno and	_oserr.

 Date		Author		Comments
 ----		------		--------
 02/09/1991	P.J. Taylor	Original.
 15/09/1991	P.J. Taylor	Amended.

 1.04
 ----
 07/03/1998	A.D.Ives	Source code reformatted.

*/

#include <sys/types.h>
#include <errno.h>
#include <qdos.h>
#include <debug.h>


long jobstrt(jobid_t jobid, int	nowait,	long *retcode) {

 /* jobid	ID of Job that is to be	started				*/
 /* nowait	True if	shouldn't wait for completion                   */
 /* retcode	Return code from job (only set for successful exec)	*/

  char *jaddr ;			/* Start of Job				*/
  struct JOBHEADER *jobhdr ;	/* Header for job being	added to	*/
  jobid_t jid, top ;		/* Parameters for mt_inf		*/
  long prior ,			/* Priority of found Job		*/
       res ;			/* Return code to be returned		*/

  errno= 0 ;
  _oserr= 0 ;

  jid= jobid ;
  top= jobid ;

  /* Check Job exists */

  if ( mt_jinf(&jid, &top, &prior, &jaddr) )
    errno= EOSERR ;

  if ( ! errno ) {

    /* Check Job not started */

    jobhdr= (struct JOBHEADER *) ( jaddr - sizeof( struct JOBHEADER )) ;

    if ( prior != 0 || jobhdr->jb_start	!= (long) jobhdr->jb_regs.jb_PC	) {
      _oserr= ERR_NJ ;
      errno= EOSERR ;
    }
  }

  if ( !errno )	{
    if ( nowait	) {
      if ( mt_activ(jobid, _def_priority, 0L) )
	errno= EOSERR ;
    }
    else {
      res= mt_activ(jobid, _def_priority, -1L) ;
      if ( _oserr )
	errno= EOSERR ;
      else
	*retcode= res ;
    }
  }

  if ( errno )
    return -1 ;
  else
    return 0 ;
}
