/*

 misc_c

 Support functions

 Date		Author		Comments
 ----		------		--------
 09/03/1998	A.D.Ives	New module added for v1.05

 1.06
 ----
 10/03/1998	A.D.Ives	RootDir(), DirChange().	UpDir()	and SetOSDir()
				moved here from	shint_c.
				FindFile() moved here from shint_c.
				New DirNav() routine added gives support for
				the "\"	".\" and "..\" prefixes	on any
				filename and lets FindFile() force a search
				in the current,	root, or parent	directory
				respectively.
				NavName() added	- short	form of	DirNav().
				Date(),	Time() added.

 11/03/1998			MediumInfo() added.

 1.09
 ----
 26/04/1998			GetOSDir() added.

*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <qdos.h>
#include <time.h>
#include "libpjt.h"

int ShellOpt(char *name) {

  /* Is	a Shell	option set? */

  int res ;
  char *opts ;

  res= FALSE ;
  opts=	getenv("SHELL_OPTS") ;
  if ( opts ) {
    res= ( strstr(opts,	name) != NULL )	;
  }
  return res ;
}

int IsNewC68(char *addr) {

  /* Is	a loaded job a 'new' C68 program (probably)? */

  unsigned char	Temp[10] ;

  Temp[0]= 0x4A	;
  Temp[1]= 0xFB	;
  Temp[2]= 0 ;
  Temp[3]= 6 ;
  Temp[4]= 'C' ;
  Temp[5]= '_' ;
  Temp[6]= 'P' ;
  Temp[7]= 'R' ;
  Temp[8]= 'O' ;
  Temp[9]= 'G' ;
  return (memcmp(Temp, addr + 6, 10) ==	0 ) ;
}

char *SlashToBackslash(char *text) {

  /* Translate slashes to backslashes */

  int pos ;

  while	( (pos=	strpos(text, '/')) != -1 )
    text[pos]= '\\' ;
  return text ;
}

char *UpDir(char *Dir) {

  /* Move up one directory level in the	supplied string	*/

  char *Ptr  ;
  for (	Ptr= Dir + strlen(Dir) - 2 ; Ptr > Dir ; Ptr-- ) {
    if ( *Ptr == '_' ) {
      *(Ptr+1)=	'\0' ;
      break ;
    }
  }
  return Dir ;
}

char *RootDir(char *Dir) {

  /* Move to the root directory	level in the supplied string */

  char *Ptr ;
  for (	Ptr= Dir ; *Ptr	; Ptr++	) {
    if ( *Ptr == '_' )
      *(Ptr+1)=	'\0' ;
  }
  return Dir ;
}

char *DirChange(char *Dir, char	*Change) {

  /*

   Pre-process a directory change string.

   Checks for the special sequences \ .. ..\ and returns
   a modified string that can be passed	directly to one
   of the chxx functions.

  */

  if ( Change[0] == '\\' ) {

    /*	 CD \	*/

    RootDir(Dir) ;
    if ( Change[1] )
      strcat(Dir, Change + 1) ;
    strcpy(Change, Dir)	;
  }
  else {
    if ( strlen(Change)	> 3 && strncmp(Change, "..\\", 3) == 0 ) {

      /*  CD ..\Dir  */

      UpDir(Dir) ;
      strcat(Dir, Change + 3);
      strcpy(Change, Dir) ;
    }
    else {
      if ( strcmp(Change, "..")	== 0 ) {

	/*  CD ..  */

	strcpy(Change, UpDir(Dir)) ;
      }
    }
  }
  return Change	;
}

void SetOSDir(int which) {

  /* Set a QDOS/SMS Directory */

  char Dir[64],	**ptr, *OSDir ;
  int l, offset	;

  switch( which	) {
    case D_DATAD :
      offset= 0xB0 ;
      getcwd(Dir, 64) ;
      break ;
    case D_PROGD :
      offset= 0xAC ;
      getcpd(Dir, 64) ;
      break ;
    case D_DESTD :
      offset= 0xB4 ;
      getcdd(Dir, 64) ;
      break ;
  }
  l= strlen(Dir) ;
  if ( l < 31 )	{
    ptr= (char **) ( ( (char *)	_sys_var ) + offset ) ;
    OSDir= *ptr	;
    OSDir[0]= 0	;
    OSDir[1]= l	;
    strcpy(OSDir + 2, Dir) ;
  }
}

char *GetOSDir(char *res, int which) {

  /* Get a QDOS/SMS Directory */

  char **ptr, *OSDir ;
  int offset, len ;

  switch( which	) {
    case D_DATAD :
      offset= 0xB0 ;
      break ;
    case D_PROGD :
      offset= 0xAC ;
      break ;
    case D_DESTD :
      offset= 0xB4 ;
      break ;
  }
  ptr= (char **) ( ( (char *) _sys_var ) + offset ) ;
  OSDir= *ptr ;
  len= ((OSDir[0] << 8)	| OSDir[1]) ;
  strncpy(res, OSDir + 2, len) ;
  res[len]= 0 ;
  return res ;
}

char* DirNav(char *Name, char *Ext, char *Dir, char *LongName) {

  /*

  Process a supplied filename, extension and directory for the
  'special' prefix characters for directory navigation.

  */

  LongName[0]= '\0' ;
  SlashToBackslash(Name) ;
  if ( strlen(Name) > 1	&& Name[0] == '\\' ) {

    /* The \ prefix forces the root directory */

    strcpy(LongName, Dir) ;
    RootDir(LongName) ;
    strcat(LongName, Name + 1) ;
    strcat(LongName, Ext) ;
    return LongName ;
  }

  if ( strlen(Name) > 2	&& Name[0] == '.' && Name[1] ==	'\\' ) {

    /* The .\ prefix forces the	current	directory */

    strcpy(LongName, Dir) ;
    strcat(LongName, Name + 2) ;
    strcat(LongName, Ext) ;
    return LongName ;
  }

  if ( strlen(Name) > 3	&& strncmp(Name, "..\\", 3) == 0) {

    /* The ..\ prefix forces the parent	directory */

    strcpy(LongName, Dir) ;
    UpDir(LongName) ;
    strcat(LongName, Name + 3) ;
    strcat(LongName, Ext) ;
    return LongName ;
  }
  return NULL ;
}

char* NavName(char *Name) {

  /*

    This is a short form of DirNav(), which returns the	modified filename.
    Very useful	for quickly processing filenames before	they are passed
    to other functions.	Assumes	the extension is already present.

    WARNING: Name must point to	an area	that has free space available to
    take the (possibly)	extended name or there'll be trouble!

  */

  char Dir[64],	LongName[128] ;

  getcwd(Dir, 64) ;
  if ( DirNav(Name, "\0", Dir, LongName) != NULL )
    strcpy(Name, LongName) ;
  return Name ;
}

void FindFile(chanid_t *Chan, char *Name, char *Ext, char *LongName, char *ProgDir, char *DataDir) {

  /*

  Try to find a	file in	the default directories, appending the supplied
  extension.

  If success, the full name of the file	will be	left in	LongName, else
  LongName points to an	empty string.

  If failure, sets errno and _oserr accordingly.

  */

  int Length, DirLength	;

  /* First, check for the 'special' prefixes */

  if ( DirNav(Name, Ext, DataDir, LongName) != NULL ) {
    chanopn(Chan, LongName, -1L, OLD_SHARE) ;
    return ;
  }

  LongName[0]= '\0' ;
  Length= strlen(Name) + strlen(Ext) ;
  if ( Length <= MAXNAMELEN ) {
    strcpy(LongName, Name) ;
    strcat(LongName, Ext) ;
    DirLength= strlen(ProgDir) ;
    if ( chanopn(Chan, LongName, -1L, OLD_SHARE) && (Length + DirLength) <= MAXNAMELEN ) {
      strcpy(LongName, ProgDir)	;
      strcat(LongName, Name ) ;
      strcat(LongName, Ext) ;
      DirLength= strlen(DataDir) ;
      if ( chanopn(Chan, LongName, -1L,	OLD_SHARE) && (Length +	DirLength) <= MAXNAMELEN ) {
	strcpy(LongName, DataDir) ;
	strcat(LongName, Name) ;
	strcat(LongName, Ext) ;
	chanopn(Chan, LongName,	-1L, OLD_SHARE)	;
      }
    }
  }
}

char *Date(char	*buff) {

  /* Return the	current	date as	a character string */

  time_t t ;
  char *tm ;

  t= time(&t) ;
  tm= ctime(&t)	;
  strncpy(buff,	tm, 10)	;
  buff[10]= ' '	;
  strncpy(buff + 11, tm	+ 20, 4) ;
  buff[15]= '\0' ;
  return buff ;
}

char *Time(char	*buff) {

  /* Return the	current	time as	a character string */

  time_t t ;
  char *tm ;

  t= time(&t) ;
  tm= ctime(&t)	;
  strncpy(buff,	tm + 11, 8) ;
  buff[8]= '\0'	;
  return buff ;
}

int MediumInfo(	unsigned long *alloc,
		unsigned long *Free,
		unsigned long *total,
		char *name ) {

  /* Get information about the medium currently	specified as the Data Default */

  int res ;
  char Dir[64] ;
  chanid_t InfoCh ;
  ext_mdinf_t Info ;

  getcwd(Dir, 64) ;
  InfoCh= res= io_open(Dir, DIROPEN) ;
  if ( InfoCh >= 0 ) {
    if ( (res= fs_xinf(InfoCh, -1L, &Info)) == 0 ) {
      *alloc= Info.xm_alloc ;
      *Free= Info.xm_free ;
      *total= Info.xm_total ;
      qlstr_to_c(name, &Info.xm_name.m_name) ;
    }
    else {
      *name= '\0' ;
      *alloc= *Free= *total= 0 ;
    }
    io_close(InfoCh) ;
  }
  return res ;
}
