/*

 setcon_c

 Set all of a CON's parameters from a string

 Date		Author		Comments
 ----		------		--------
 12/03/1998	A.D.Ives	New module for v1.06
 13/03/1998	D.C.Gilham	made code more generic
				used new routine strout
				instead	of printf in MoveCon
 14/03/1998	D.C.Gilham	replaced sscanf	with do	while
				and dedicated conversion function
				prevents system	crashes.
				also changed to	if value of param < 0
				stored value used.
 08/07/2001	D.C.Gilham	Changed	type of	key to unsigned	char
				this removed a lot of level 2 warnings
				about vaules in	case statments being out
				of range. Also did apropriate cast in io_fbyte
				call.

*/

#include <stdio.h>
#include <stdlib.h>
#include <qdos.h>
#include <qptr.h>
#include <string.h>
#include "libpjt.h"
#include "keys.h"

extern int strout(chanid_t chan,char * string,timeout_t	tmout);
extern char * strol(char *,int *);

#define	D_WIDTH	  484
#define	D_HEIGHT  215
#define	D_XORG	  16
#define	D_YORG	  30
#define	D_BWIDTH  1
#define	D_BCOLOUR 2
#define	D_PAPER	  0
#define	D_INK	  7
#define	D_YSHAD	  0
#define	D_XSHAD	  0

static int curr_w= -1,
	   curr_h= -1,
	   curr_x= -1,
	   curr_y= -1,
	   curr_b_wid= -1,
	   curr_b_col= -1,
	   curr_paper= -1,
	   curr_ink= -1,
	   curr_xshad= -1,
	   curr_yshad= -1 ;

void SetCon(chanid_t ch, char *def) {

  /*

    Set	a CONsole definition from the description in a string.

  */

  QLRECT_t ConDef ;
  char * tmptr=def;
  WM_wsiz_t size ;
  int w= -1,
      h= -1,
      x= -1,
      y= -1,
      b_wid= -1,
      b_col= -1,
      paper= -1,
      ink= -1,
      xshad= -1,
      yshad= -1	;

  if ( def != NULL && *def ) {
    if ( stricmp(def, "-m") == 0 ) {
      MoveCon(ch) ;
      return ;
    }
#if 0
    sscanf(def,	"%d %d %d %d %d %d %d %d %d %d",
		 &w, &h, &x, &y, &b_wid, &b_col, &paper, &ink, &xshad, &yshad );

#else
    do {
       if(!(tmptr=strol(tmptr,&w)))
       break;
       if(!(tmptr=strol(tmptr,&h)))
       break;
       if(!(tmptr=strol(tmptr,&x)))
       break;
       if(!(tmptr=strol(tmptr,&y)))
       break;
       if(!(tmptr=strol(tmptr,&b_wid)))
       break;
       if(!(tmptr=strol(tmptr,&b_col)))
       break;
       if(!(tmptr=strol(tmptr,&paper)))
       break;
       if(!(tmptr=strol(tmptr,&ink)))
       break;
       if(!(tmptr=strol(tmptr,&xshad)))
       break;
       if(!(tmptr=strol(tmptr,&yshad)))
       break;
       break;
       } while(1);
#endif
#if 0
    if ( w == -1 ) w= curr_w ;
    if ( h == -1 ) h= curr_h ;
    if ( x == -1 ) x= curr_x ;
    if ( y == -1 ) y= curr_y ;
    if ( b_wid == -1 ) b_wid= curr_b_wid ;
    if ( b_col == -1 ) b_col= curr_b_col ;
    if ( paper == -1 ) paper= curr_paper ;
    if ( ink ==	-1 ) ink= curr_ink ;
    if ( xshad == -1 ) xshad= curr_xshad ;
    if ( yshad == -1 ) yshad= curr_yshad ;
#else
    if ( w < 0 ) w= curr_w ;
    if ( h < 0 ) h= curr_h ;
    if ( x < 0 ) x= curr_x ;
    if ( y < 0 ) y= curr_y ;
    if ( b_wid < 0 ) b_wid= curr_b_wid ;
    if ( b_col < 0 ) b_col= curr_b_col ;
    if ( paper < 0 ) paper= curr_paper ;
    if ( ink < 0 ) ink=	curr_ink ;
    if ( xshad < 0 ) xshad= curr_xshad ;
    if ( yshad < 0 ) yshad= curr_yshad ;
#endif


  }
  else {

    /* No parameter string supplied, so	use the	defaults */

    w=	   D_WIDTH ;
    h=	   D_HEIGHT ;
    x=	   D_XORG ;
    y=	   D_YORG ;
    b_wid= D_BWIDTH ;
    b_col= D_BCOLOUR ;
    paper= D_PAPER ;
    ink=   D_INK ;
    xshad= D_XSHAD ;
    yshad= D_YSHAD ;
  }

  ConDef.q_width=  size.xsize= w ;
  ConDef.q_height= size.ysize= h ;
  ConDef.q_x=	   size.xorg=  x ;
  ConDef.q_y=	   size.yorg=  y ;

  iop_outl(ch, -1, xshad, yshad, 0, &size) ;

  sd_wdef(ch,
	  -1,
	  b_col,
	  b_wid,
	  &ConDef) ;
  sd_setst(ch, -1, paper) ;
  sd_setpa(ch, -1, paper) ;
  sd_setin(ch, -1, ink)	;
  sd_clear(ch, -1) ;

  /* Save the new settings that	we've just adopted */


  curr_w= w ;
  curr_h= h ;
  curr_x= x ;
  curr_y= y ;
  curr_b_wid= b_wid ;
  curr_b_col= b_col ;
  curr_paper= paper ;
  curr_ink= ink	;
  curr_xshad= xshad ;
  curr_yshad= yshad ;
}

void MoveCon(chanid_t Chan) {

  /*

    Allow the user to move a console window around the screen.

    Note that this routine is not totally generic (yet)	because
    it does printf() calls to give the user directions,	and we
    really want	them going to the supplied QDOS	Channel	ID.

  */

  unsigned  char Key ;
  int Changed ;

  sd_clear(Chan, -1) ;
  sd_cure(Chan,	-1) ;
  Key= 0 ;
  while	( Key != K_ESC && Key != K_ENTER ) {
#if 0
    printf("Use Cursor Keys to move the console window.\n") ;
    printf("[SHIFT] plus a cursor key moves by character.\n") ;
    printf("[CTRL] plus a cursor key changes the size.\n") ;
    printf("Press [ENTER] to save, or [ESC] to reset.\n") ;
#else
    strout(Chan,"Use Cursor Keys to move the console window.\n",-1) ;
    strout(Chan,"[SHIFT] plus a cursor key moves by character.\n",-1) ;
    strout(Chan,"[CTRL] plus a cursor key changes the size.\n",-1) ;
    strout(Chan,"Press [ENTER] to save, or [ESC] to reset.\n",-1) ;
#endif
    Changed= FALSE ;
    io_fbyte(Chan, -1,(char*) &Key) ;
    switch ( Key ) {
    case K_UP :
      if ( curr_y > 0 )	{
	curr_y-- ;
	Changed	= TRUE ;
      }
      break ;
    case K_DOWN	:
      curr_y++ ;
      Changed= TRUE ;
      break ;
    case K_LEFT	:
      if ( curr_x > 2) {
	curr_x-= 2 ;
	Changed= TRUE ;
      }
      break ;
    case K_RIGHT :
      curr_x+= 2 ;
      Changed= TRUE ;
      break ;
    case K_SH_UP :
      if ( curr_y > 10 ) {
	curr_y-= 10 ;
	Changed	= TRUE ;
      }
      break ;
    case K_SH_DOWN :
      curr_y+= 10 ;
      Changed= TRUE ;
      break ;
    case K_SH_LEFT :
      if ( curr_x > 6 )	{
	curr_x-= 6 ;
	Changed= TRUE ;
      }
      break ;
    case K_SH_RIGHT :
      curr_x+= 6 ;
      Changed= TRUE ;
      break ;
    case K_CT_DOWN :
      curr_h++ ;
      Changed= TRUE ;
      break ;
    case K_CT_UP :
      if ( curr_h > 0 )	{
	curr_h-- ;
	Changed= TRUE ;
      }
      break ;
    case K_CT_RIGHT :
      curr_w+= 2 ;
      Changed= TRUE ;
      break ;
    case K_CT_LEFT :
      if ( curr_w > 2 )	{
	curr_w-= 2 ;
	Changed= TRUE ;
      }
      break ;
    case K_CS_DOWN :
      curr_h+= 10 ;
      Changed= TRUE ;
      break ;
    case K_CS_UP :
      if ( curr_h > 1 )	{
	curr_h-= 10 ;
	Changed= TRUE ;
      }
      break ;
    case K_CS_RIGHT :
      curr_w+= 6 ;
      Changed= TRUE ;
      break ;
    case K_CS_LEFT :
      if ( curr_w > 6) {
	curr_w-= 6 ;
	Changed= TRUE ;
      }
      break ;
    }
    if ( Changed )
      SetCon(Chan, "-1 -1 -1 -1") ;
    else
      sd_clear(Chan, -1) ;
  }
  if ( Key == K_ESC )
    SetCon(Chan, NULL) ;
  sd_curs(Chan,	-1) ;

}
