/*
 *   c o n s e t u p _ t i t l e
 *
 *  This routine is one	of the standard	C68 console set-up routines
 *  (although not the default).	  It sets up a window plus title bar
 *  along the style used by QJUMP Pointer Environment programs.	 It
 *  also sets up a "Outline Window" if pointer Environment is in use.
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  20 Aug 94	DJW   -	Added qptr.h header file as iop_outl() now in this
 *			file with other	qptr routiones.
 *  15 Jul 97	DCG   -	Severly	shortened and simplified
 *			made allowance for fact	that on
 *			QXL with SMQ and no pointer env
 *			larger screens are possible
 *			see file autowin based on Mark Knights work
 *			for details
 *
 * 13 Jan  98  DCG    -	checked	that size of window will fit in
 *			and adjusted as	needed.
 *			if one dimension too big whole screen used
 *			not what was intended !
 *
 * 11 Feb  98  DCG    -	new version which simply centres window	in screen
 *
 *
 * 6  Sep  98 DCG   - now uses fmaxsize	to determine maximum size of window
 *
 */

#define	__LIBRARY__

#include <qdos.h>
#include <qptr.h>
#include <string.h>
extern int fmaxsize(chanid_t,struct QLRECT*);

void consetup_title4(chanid_t chanid, struct WINDOWDEF*	condetails)
{
    struct   QLRECT winlim;

    /*
     *	The vertical border width is ALWAYS 2 *	the horizontal
     *	border width (height ?), because we measure the	width in
     *	pixel-coordinate units.	In MODE	8, pixels have a width of 2 !
     *
     *	Erling Jacobsen	( aka EAJ )
     */
    /* Odd width is a bad idea */
    condetails->width =	(short)(condetails->width & ~1);
    /*
     *	Get the	possible size of the window, use old QL	size, if necc.
     *
     *	Erling Jacobsen
     */

       fmaxsize(chanid,&winlim);

	/* Check that window is	OK */
	if( condetails->height > winlim.q_height )
	    condetails->height = winlim.q_height;

	if( condetails->width >	winlim.q_width )
	    condetails->width =	winlim.q_width;

	if( condetails->x_origin < winlim.q_x )
	    condetails->x_origin = winlim.q_x;

	if( condetails->y_origin < winlim.q_y )
	    condetails->y_origin = winlim.q_y;

	    /* following should	centre window on screen	*/
	condetails->x_origin =winlim.q_x+(winlim.q_width-condetails->width)>>1;
	condetails->y_origin =winlim.q_y+(winlim.q_height-condetails->height)>>1;

    /*
     *	For some reason, sd_wdef works better, if called AFTER iop_outl. EAJ
     *	This should make outline greater than or equal to window size.	 DCG
     *
     */
    (void) iop_outl(chanid, (timeout_t)-1, 0, 0, 0,(struct WM_wsiz *) &winlim);

    (void) sd_wdef(chanid, (timeout_t)-1, condetails->border_colour,
					  condetails->border_width,
					  (struct QLRECT *)&condetails->width);

    /*
     *	Now set	up standard working window.
     *	It is defined with no border to	fit
     *	inside the original one	opened above.
     */

    (void) sd_setpa(chanid, (timeout_t)-1, condetails->paper);
    (void) sd_setst(chanid, (timeout_t)-1, condetails->paper);
    (void) sd_setin(chanid, (timeout_t)-1, condetails->ink);
    (void) sd_bordr(chanid, (timeout_t)-1,condetails->border_colour,condetails->border_width);
    (void) sd_clear(chanid, (timeout_t)-1);

    return;
}
